package tv.younify.sdk.connectapp

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.LifecycleCoroutineScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import tv.younify.sdk.connect.StreamingCategory
import tv.younify.sdk.connect.StreamingContent
import tv.younify.sdk.connect.StreamingService
import tv.younify.sdk.connectapp.databinding.YounifyCategoryRowBinding

internal class YounifyCategoriesAdapter(private val lifecycleScope: LifecycleCoroutineScope, private val onTapListener: YounifyCategoryItemsAdapter.OnTapListener, private val imageRequestListener: ImageRequestListener): RecyclerView.Adapter<YounifyCategoriesAdapter.CategoryViewHolder>() {
    private var categories = mutableListOf<StreamingCategory>()
    private var categoryViewsMap = mutableMapOf<StreamingCategory, CategoryViewHolder>()
    private var categoriesData = mutableListOf<YounifyCategoryServices>()
    private var isLoading = false

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): CategoryViewHolder {
        val binding = YounifyCategoryRowBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return CategoryViewHolder(binding)
    }

    override fun onBindViewHolder(holder: CategoryViewHolder, position: Int) {
        categoryViewsMap[categories[position]] = holder
        holder.bind(categories[position], isLoading, onTapListener, imageRequestListener)
        val categoryData = categoriesData.firstOrNull { it.category == categories[position] }
        categoryData?.services?.forEach { holder.addContentItems(it.content, it.service) }
    }

    override fun getItemCount(): Int {
        return categories.size
    }

    @SuppressLint("NotifyDataSetChanged")
    fun setCategories(categories: List<StreamingCategory>) {
        this.categories = categories.toMutableList()
        notifyDataSetChanged()
    }

    fun addCategoryContentItems(category: StreamingCategory, service: StreamingService, items: List<StreamingContent>) {
        var categoryData = categoriesData.firstOrNull { it.category == category }
        if (categoryData == null) {
            categoryData = YounifyCategoryServices(category)
            categoriesData.add(categoryData)
        }

        var serviceData = categoryData.services.firstOrNull { it.service.name == service.name }
        if (serviceData == null) {
            serviceData = YounifyServiceContent(service)
            categoryData.services.add(serviceData)
        }

        serviceData.content.addAll(items)

        val view = categoryViewsMap[category]
        view?.addContentItems(items, service)
    }

    fun startLoading() {
        isLoading = true
    }

    fun finishLoading() {
        isLoading = false
        categoryViewsMap.forEach {
            it.value.finishLoading()
        }
    }

    inner class CategoryViewHolder(private val binding: YounifyCategoryRowBinding) : RecyclerView.ViewHolder(binding.root) {
        private var adapter: YounifyCategoryItemsAdapter? = null

        fun bind(category: StreamingCategory, isLoading: Boolean, tapListener: YounifyCategoryItemsAdapter.OnTapListener, imageRequestListener: ImageRequestListener) {
            binding.categoryTitle.text = category.name
            if (isLoading)
                binding.activityIndicator.visibility = View.VISIBLE
            else
                binding.activityIndicator.visibility = View.GONE

            binding.categoryItemsView.layoutManager = LinearLayoutManager(itemView.context, LinearLayoutManager.HORIZONTAL,false)
            adapter = YounifyCategoryItemsAdapter(lifecycleScope, tapListener, imageRequestListener)
            binding.categoryItemsView.adapter = adapter
        }

        fun addContentItems(items: List<StreamingContent>, service: StreamingService) {
            finishLoading()
            adapter?.addCategoryItems(items, service)
        }

        fun finishLoading() {
            binding.activityIndicator.visibility = View.GONE
        }
    }
}
