package tv.younify.sdk.connectapp

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.util.Log
import android.util.LruCache
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.lifecycle.LifecycleCoroutineScope
import androidx.recyclerview.widget.RecyclerView
import kotlinx.coroutines.launch
import tv.younify.sdk.connect.StreamingContent
import tv.younify.sdk.connect.StreamingService
import tv.younify.sdk.connectapp.databinding.YounifyCategoryContentItemViewBinding
import kotlin.coroutines.cancellation.CancellationException

internal class YounifyCategoryItemsAdapter() : RecyclerView.Adapter<YounifyCategoryItemsAdapter.ContentItemViewHolder>() {
    private var lifecycleScope: LifecycleCoroutineScope? = null
    private var onTapListener: OnTapListener? = null
    private var imageRequestListener: ImageRequestListener? = null
    private var categoryItems = mutableListOf<StreamingContent>()
    private var servicesMap = mutableMapOf<StreamingContent, StreamingService>()

    private var imagesCache: LruCache<String, Bitmap>? = null
    private var maxMemory = (Runtime.getRuntime().maxMemory() / 1024).toInt()
    private var cacheSize: Int = maxMemory / 8

    constructor(lifecycleScope: LifecycleCoroutineScope, tapListener: OnTapListener, imageRequestListener: ImageRequestListener) : this() {
        this.lifecycleScope = lifecycleScope
        this.onTapListener = tapListener
        this.imageRequestListener = imageRequestListener
        this.imagesCache = LruCache<String, Bitmap>(cacheSize)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ContentItemViewHolder {
        val binding = YounifyCategoryContentItemViewBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return ContentItemViewHolder(binding)
    }

    override fun onBindViewHolder(holder: ContentItemViewHolder, position: Int) {
        val item = categoryItems[position]
        val service = servicesMap[item]
        holder.bind(item, service!!)
    }

    override fun getItemCount(): Int {
        return categoryItems.size
    }

    fun addCategoryItems(items: List<StreamingContent>, service: StreamingService) {
        items.forEach { servicesMap[it] = service }
        val start = this.categoryItems.count()
        val end = start + items.count()
        this.categoryItems.addAll(items)
        notifyItemRangeInserted(start, end)
    }

    inner class ContentItemViewHolder(private val binding: YounifyCategoryContentItemViewBinding) : RecyclerView.ViewHolder(binding.root) {
        private var streamingContent: StreamingContent? = null
        private var streamingService: StreamingService? = null

        fun bind(item: StreamingContent, service: StreamingService) {
            streamingContent = item
            streamingService = service
            binding.name.text = item.title

            lifecycleScope?.launch {
                try {
                    val thumb = getBitmapFromMemCache(item.smallThumbnailUrl)
                    if (thumb != null)
                        binding.image.setImageBitmap(thumb)
                    else {
                        val itemImageData = imageRequestListener?.fetchImage(item.smallThumbnailUrl!!)
                        val itemImageBmp = BitmapFactory.decodeByteArray(itemImageData, 0, itemImageData?.size!!)
                        addBitmapToMemoryCache(item.smallThumbnailUrl, itemImageBmp)
                        binding.image.setImageBitmap(itemImageBmp)
                    }

                    val overlay = getBitmapFromMemCache(service.overlayThumbnailUrl!!)
                    if (overlay != null)
                        binding.imageService.setImageBitmap(overlay)
                    else {
                        val serviceImageData = imageRequestListener?.fetchImage(service.overlayThumbnailUrl!!)
                        val serviceImageBmp = BitmapFactory.decodeByteArray(serviceImageData, 0, serviceImageData?.size!!)
                        addBitmapToMemoryCache(service.overlayThumbnailUrl, serviceImageBmp)
                        binding.imageService.setImageBitmap(serviceImageBmp)
                    }
                }
                catch (ex: CancellationException) {
                    throw ex
                }
                catch (cause: Throwable) {
                    Log.e("YounifyCategoryItemsAdapter", cause.message!!)
                }
            }

            itemView.setOnClickListener { onTapListener?.itemClick(streamingContent!!, streamingService!!) }
        }
    }

    class OnTapListener(val tapListener: (content: StreamingContent, service: StreamingService) -> Unit) {
        fun itemClick(content: StreamingContent, service: StreamingService) = tapListener(content, service)
    }

    private fun addBitmapToMemoryCache(key: String?, bitmap: Bitmap?) {
        if (getBitmapFromMemCache(key) == null)
            imagesCache?.put(key, bitmap)
    }

    private fun getBitmapFromMemCache(key: String?): Bitmap? {
        return imagesCache?.get(key)
    }
}
