package tv.younify.sdk.connectapp

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import tv.younify.sdk.connect.StreamingService
import tv.younify.sdk.connectapp.databinding.AdapterServicesBinding

internal class YounifyServicesAdapter(private val onTapListener: OnTapListener) : ListAdapter<StreamingService, ServiceViewHolder>(ServicesDiffUtil) {
    companion object ServicesDiffUtil : DiffUtil.ItemCallback<StreamingService>() {
        override fun areItemsTheSame(oldItem: StreamingService, newItem: StreamingService): Boolean {
            return oldItem == newItem
        }

        override fun areContentsTheSame(oldItem: StreamingService, newItem: StreamingService): Boolean {
            return oldItem.isSameService(newItem)
        }
    }

    private var services = mutableListOf<StreamingService>()

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ServiceViewHolder {
        val binding = AdapterServicesBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return ServiceViewHolder(binding)
    }

    override fun onBindViewHolder(holder: ServiceViewHolder, position: Int) {
        val service = services[position]
        holder.service.text = service.name
        holder.account.text = service.link?.username
        holder.profile.text = service.link?.profileName

        Glide.with(holder.itemView.context)
            .load(service.smallThumbnailUrl)
            .into(holder.img)

        holder.itemView.setOnClickListener { onTapListener.itemClick(service) }
        holder.fetch.setOnClickListener { onTapListener.fetchClick(service) }
        holder.unlink.setOnClickListener { onTapListener.unlinkClick(service) }

        if (service.link != null) {
            holder.fetch.visibility = View.VISIBLE
            holder.unlink.visibility = View.VISIBLE
        } else {
            holder.fetch.visibility = View.GONE
            holder.unlink.visibility = View.GONE
        }

        if (service.link?.isBroken == true)
            holder.imgBroken.visibility = View.VISIBLE
        else
            holder.imgBroken.visibility = View.GONE
    }

    override fun getItemCount(): Int {
        return services.size
    }

    @SuppressLint("NotifyDataSetChanged")
    fun setServiceItems(services: List<StreamingService>) {
        this.services = services.toMutableList()
        notifyDataSetChanged()
    }

    fun updateService(service: StreamingService) {
        val index = services.indexOfFirst { it.name == service.name }
        if (index >= 0) {
            services[index] = service
            notifyItemChanged(index)
        }
    }

    enum class TapListenerAction {
        ItemClick,
        FetchClick,
        UnlinkClick
    }

    class OnTapListener(val tapListener: (service: StreamingService, action: TapListenerAction) -> Unit) {
        fun itemClick(service: StreamingService) = tapListener(service, TapListenerAction.ItemClick)
        fun fetchClick(service: StreamingService) = tapListener(service, TapListenerAction.FetchClick)
        fun unlinkClick(service: StreamingService) = tapListener(service,TapListenerAction.UnlinkClick)
    }
}

class ServiceViewHolder(binding: AdapterServicesBinding) : RecyclerView.ViewHolder(binding.root) {
    val img = binding.imgService
    val imgBroken = binding.imgBrokenService
    val service = binding.textService
    val account = binding.textAccount
    val profile = binding.textProfile
    val fetch = binding.fetchService
    val unlink = binding.unlinkService
}
